% drawSpaceBounds(sz,spaceDepthCm,fc,cc)
% 
% Draw the bounds around the space visible by the camera
%
% sz -- size of the image [ni nj] or [ny nx]
% spaceDepthCm -- farthest distance "visible", in cm
% fc - vector of focal lengths in j and i directions.
%       that is, fc = [fc_j fc_i] == [fc_x fc_y]
% cc - camera center vector
%       pixel location of the camera center [cc_x cc_y] == [cc_j cc_i]
function drawSpaceBounds(sz,spaceDepthCm,fc,cc)
    [ii,jj] = ndgrid([1 sz(1)],[1 sz(2)]);
    ii([4 3]) = ii([3 4]);
    jj([4 3]) = jj([3 4]);
    v = reprojectPoint(jj(:)',ii(:)',fc,cc);
    v = v*spaceDepthCm;
    lines = [v zeros(size(v));
    v(:,2:end), v(:,1), v];
    plot3( [lines(3,:); lines(6,:)], [lines(1,:); lines(4,:)], [lines(2,:); lines(5,:)],'r-');
    
    ylabel('x (cm)');
    zlabel('y (cm)');
    xlabel('z (cm)');
    set(gca,'ydir','reverse')
    set(gca,'xdir','normal')
    set(gca,'zdir','reverse')
    axis equal
end

